<?php
/**
 * @version    1.0.2
 * @package    com_gadecisions
 * @author     Glenn Arkell <glenn@glennarkell.com.au>
 * @copyright  2021 Glenn Arkell
 * @license    GNU General Public License version 2 or later; see LICENSE.txt
 */

namespace GlennArkell\Component\Gadecisions\Site\Model;

// No direct access.
defined('_JEXEC') or die;

use \Joomla\CMS\Factory;
use \Joomla\CMS\Language\Text;
use \Joomla\CMS\MVC\Model\ListModel;
use \Joomla\Component\Fields\Administrator\Helper\FieldsHelper;
use \Joomla\CMS\Helper\TagsHelper;
use \Joomla\CMS\Layout\FileLayout;
use \Joomla\Database\ParameterType;
use \Joomla\Utilities\ArrayHelper;
use \GlennArkell\Component\Gadecisions\Administrator\Helper\GadecisionsHelper;

/**
 * Methods supporting a list of records.
 * @since  1.6
 */
class MotionsModel extends ListModel
{
	/**
	 * Constructor.
	 * @param   array  $config  An optional associative array of configuration settings.
	 * @see        JController
	 * @since      1.6
	 */
	public function __construct($config = array())
	{
		if (empty($config['filter_fields']))
		{
			$config['filter_fields'] = array(
				'id', 'a.id',
				'ordering', 'a.ordering',
				'state', 'a.state',
				'created_date', 'a.created_date',
				'motion', 'a.motion',
				'motion_date', 'a.motion_date',
				'cat_id', 'a.cat_id', 'cat_id_name',
				'group_id', 'a.group_id', 'group_id_name',
			);
		}

		parent::__construct($config);
	}

	/**
	 * Method to auto-populate the model state.
	 * Note. Calling getState in this method will result in recursion.
	 * @param   string  $ordering   Elements order
	 * @param   string  $direction  Order direction
	 * @return void
	 * @throws Exception
	 * @since    1.6
	 */
	protected function populateState($ordering = null, $direction = null)
	{
		$app  = Factory::getApplication();
		$list = $app->getUserState($this->context . '.list');

		$ordering  = isset($list['filter_order'])     ? $list['filter_order']     : null;
		$direction = isset($list['filter_order_Dir']) ? $list['filter_order_Dir'] : null;

		if(empty($ordering)) {
			$ordering = $app->getUserStateFromRequest($this->context . '.filter_order', 'filter_order', $app->get('filter_order'));
			if (!in_array($ordering, $this->filter_fields)) {
				$ordering = "a.motion_date";
			}
			$this->setState('list.ordering', $ordering);
		}
		if(empty($direction)) {
			$direction = $app->getUserStateFromRequest($this->context . '.filter_order_Dir', 'filter_order_Dir', $app->get('filter_order_Dir'));
			if (!in_array(strtoupper($direction ?? ''), array('ASC', 'DESC', ''))) {
				$direction = "DESC";
			}
			$this->setState('list.direction', $direction);
		}

		$list['limit']     = $app->getUserStateFromRequest($this->context . '.list.limit', 'limit', $app->get('list_limit'), 'uint');
		$list['start']     = $app->input->getInt('start', 0);
		$list['ordering']  = $ordering;
		$list['direction'] = $direction;
		
		$app->setUserState($this->context . '.list', $list);
		//$app->input->set('list', null);   // if you use this, list always defaults to Global

        // List state information.
        parent::populateState($ordering, $direction);

        $context = $this->getUserStateFromRequest($this->context.'.filter.search', 'filter_search');
        $this->setState('filter.search', $context);
        $status = $this->getUserStateFromRequest($this->context.'.filter.state', 'filter_state');
        $this->setState('filter.state', $status);
        $catid = $this->getUserStateFromRequest($this->context.'.filter.cat_id', 'filter_cat_id');
        $this->setState('filter.cat_id', $catid);
        $groupid = $this->getUserStateFromRequest($this->context.'.filter.group_id', 'filter_group_id');
        $this->setState('filter.group_id', $groupid);

	}

	/**
	 * Build an SQL query to load the list data.
	 * @return   JDatabaseQuery
	 * @since    1.6
	 */
	protected function getListQuery()
	{
        // check for menu params
        $menuitem = Factory::getApplication()->getMenu()->getActive();
        $listgroup = $menuitem->getParams()->get('menu_group_id');
        $listmotion = $menuitem->getParams()->get('menu_motion_id');
        $user = GadecisionsHelper::getSpecificUser();
        $canAdmin = $user->authorise('core.admin', 'com_gadecisions');
        // Create a new query object.
        $db    = $this->getDbo();
        $query = $db->getQuery(true);

        // Select the required fields from the table.
        $query->select( $this->getState( 'list.select', 'DISTINCT a.*' ) );

        $query->from('`#__gadecisions_motions` AS a');

		// Join over the users for the checked out user.
		$query->select('uc.name AS uEditor');
		$query->join('LEFT', '#__users AS uc ON uc.id=a.checked_out');

		// Join over the created by field 'created_by'
		$query->select('created_by.name AS created_by_name');
		$query->join('LEFT', '#__users AS created_by ON created_by.id = a.created_by');

		// Join over the created by field 'modified_by'
		$query->select('modified_by.name AS modified_by_name');
		$query->join('LEFT', '#__users AS modified_by ON modified_by.id = a.modified_by');

		// Join over the user field 'mover'
		$query->select('m.name AS mov_id_name');
		$query->join('LEFT', '#__users AS m ON m.id = a.mov_id');

		// Join over the user field 'seconder'
		$query->select('s.name AS sec_id_name');
		$query->join('LEFT', '#__users AS s ON s.id = a.sec_id');

		// Join over the category field 'cat_id'
		$query->select('c.title AS cat_id_name');
		$query->join('LEFT', '#__categories AS c ON c.id = a.cat_id');

		// Join over the group field 'group_id'
		$query->select('g.title AS group_id_name');
		$query->join('LEFT', '#__usergroups AS g ON g.id = a.group_id');


		// Filter by published state
        $status = $this->getState('filter.state');
		if (is_numeric($status)) {
			$query->where('a.state = ' . (int) $status);
		} elseif ($status === '*') {
			// show all and don't filter on status
		} else {
			if (!GadecisionsHelper::getSpecificUser()->authorise('core.edit', 'com_gadecisions')) {
				$query->where('a.state = 1');
			} else {
				$query->where('(a.state IN (0, 1))');
			}
        }

		// Filter by category
        $catid = $this->getState('filter.cat_id');
		if ($catid) {
			$query->where('a.cat_id = ' . (int) $catid);
        }

		// Filter by group
        $groupid = $this->getState('filter.group_id');
		if ($groupid && !$canAdmin) {
			$query->where('a.group_id = ' . (int) $groupid);
        }

		// Filter by menu setting of group
		if ($listgroup && !$canAdmin) {
			$query->where('a.group_id = ' . (int) $listgroup);
        }

        // Filter by search in title
        $search = $this->getState('filter.search');
        if (!empty($search)) {
            if (stripos($search, 'id:') === 0) {
                $query->where('a.id = ' . (int) substr($search, 3));
            } else {
                $search = $db->Quote('%' . $db->escape($search, true) . '%');
				$query->where('( a.motion LIKE ' . $search . ' OR s.name LIKE ' . $search . ' OR m.name LIKE ' . $search . ')');
            }
        }
            
        // Add the list ordering clause.
        $orderCol  = $this->state->get('list.ordering', "a.motion_date");
        $orderDirn = $this->state->get('list.direction', "DESC");

        if ($orderCol && $orderDirn) {
            $query->order($db->escape($orderCol . ' ' . $orderDirn));
        }

        return $query;
	}

	/**
	 * Method to get an array of data items
	 * @return  mixed An array of data on success, false on failure.
	 */
	public function getItems()
	{
		$items = parent::getItems();
		
		return $items;
	}

	/**
	 * Overrides the default function to check Date fields format, identified by
	 * "_dateformat" suffix, and erases the field if it's not correct.
	 * @return void
	 */
	protected function loadFormData()
	{
		$app              = Factory::getApplication();
		$filters          = $app->getUserState($this->context . '.filter', array());
		$error_dateformat = false;

		foreach ($filters as $key => $value) {
			if (strpos($key, '_dateformat') && !empty($value) && $this->isValidDate($value) == null) {
				$filters[$key]    = '';
				$error_dateformat = true;
			}
		}

		if ($error_dateformat) {
			$app->enqueueMessage(Text::_("COM_GADECISIONS_SEARCH_FILTER_DATE_FORMAT"), "warning");
			$app->setUserState($this->context . '.filter', $filters);
		}

		return parent::loadFormData();
	}

	/**
	 * Checks if a given date is valid and in a specified format (YYYY-MM-DD)
	 * @param   string  $date  Date to be checked
	 * @return bool
	 */
	private function isValidDate($date)
	{
		$date = str_replace('/', '-', $date);
		return (date_create($date)) ? Factory::getDate($date)->format("Y-m-d") : null;
	}
}
